/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.FilterBox");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.i18n");
dojo.requireLocalization("jazz.ui", "FilterBoxMessages");

(function() {

var messages = dojo.i18n.getLocalization("jazz.ui", "FilterBoxMessages");

/*
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 */
dojo.declare("jazz.ui.FilterBox", [dijit._Widget, dijit._Templated], {
	// summary:
	//		A text box to be used for filtering.
	//
	//      NOTE: This is provisional API. Clients using this API do so with the
	//		awareness that this class may be removed or substantially changed at
	//		future times without warning.
	// description:
	//		This is a simple input field with some additional logic to support use as a
	//		filter input box. Includes grey text to guide the user when empty, a button
	//		to clear the content when populated and an onChange handler to respond to user
	//		input.
	// value:
	//		A read only string attribute that gives the current text value.
	// onChange:
	//		A function that will be called any time the content of the input box changes.
	//		The function will be passed a single parameter which is the new value of the box.
	// enabled:
	//		A read only boolean attribute that indicates if the box is currently enabled.
	//		Though it is read only, the default can be updated in the widget constructor. To
	//		change the enabled state during the widget lifecycle, use the setEnabled function.
	// defaultText:
	//		The grey text that should appear when the filter box is empty. By default it says
	//		"Type to filter".
	// since:
	//		0.7
	
	value: "",
	onChange: null,
	enabled: true,
	defaultText: messages.filter,
	
	setEnabled: function(/* boolean */ enabled) {
		// summary:
		//		Enables or disables the box.
		// description:
		//		Can be called to enable or disable the box.
		// enabled:
		//		A boolean indicating if the box should be enabled or disabled.
		this._box.disabled = enabled ? false : true;
		this.enabled = enabled;
	},
	
	// INTERNAL PROPERTIES AND FUNCTIONS
		
	templatePath: dojo.moduleUrl("jazz.ui", "templates/FilterBox.html"),
	
	postCreate: function() {
		this._blur();
		if (!this.enabled)
			this._box.disabled = true;
	},
	
	_focus: function(event) {
		if (dojo.hasClass(this._box, "helpMessage")) {
			this._box.value = "";
			dojo.removeClass(this._box, "helpMessage");
			dojo.removeClass(this._clearLink, "hidden");
		}
	},
	
	_blur: function(event) {
		if (this._box.value.length == 0) {
			// it is possible to clear and leave the box before the filter has a chance to run
			// ensure that it runs before we change the value of the box (call will be ignored if the filter is the same)
			this._keyup();
			dojo.addClass(this._box, "helpMessage");
			dojo.addClass(this._clearLink, "hidden");
			this._box.value = this.defaultText;
		}
	},
	
	_keyup: function(event) {
		if (this.value && this.value == this._box.value.toLowerCase())
			return;
		this.value = this._box.value.toLowerCase();
		if (this.onChange)
			this.onChange(this.value);
	},
	
	_clear: function(event) {
		event.preventDefault();
		this._box.value = "";
		this._clearLink.blur();
	}

});
	
})();
