/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.internal.LoginLogoutArea");

dojo.require("dojo.cookie");
dojo.require("jazz.client.xhr");
dojo.require("jazz.app.auth");
dojo.require("jazz.util.html");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.requireLocalization("jazz.app.internal", "LoginLogoutMessages");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

dojo.declare("jazz.app.internal.LoginLogoutArea", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("jazz.app", "internal/templates/LoginLogoutArea.html"),

	config: null,	//passed into constructor
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("jazz.app.internal", "LoginLogoutMessages");
	},
	postCreate: function() {
		if(this.config && this.config.user) {
			this.setLoggedIn();
		} else {
			this.setLoggedOut();
		}
	},
	destroy: function() {
		
		if (this._userCallbackHandler) {
			this.disconnect(this._userCallbackHandler);
			delete this._userCallbackHandler;
		}
		
		if (this._userHoverMenu) {
			this._userHoverMenu.destroy();
			delete this._userHoverMenu;
		}
		
		this.inherited(arguments);
	},		
	setLoggedIn: function() {
		this._setMode("logout");
	},
	setLoggedOut: function() {
		this._setMode("login");
	},
	_setMode: function(/*String*/mode) {
		var config = this.config;
		if(mode !== "login" && mode !== "logout") {
			throw new Error("Invalid mode: " + mode);
		}
		if (this._clickEvent) this.disconnect(this._clickEvent);
		
		var buttons = this.config.buttons;
		if (dojo.isArray(buttons)) {
			for (var i = 0; i < buttons.length; i++) {
				if (buttons[i] && buttons[i].align && buttons[i].align == "right") {
					this.rightPipe.style.display = "inline";
				}
			}
		}
		if (this.config.user && (this.config.user.userId ||this.config.user.name)) {
			this.userIdWrapper.style.display = "";
		} else {
			this.userIdWrapper.style.display = "none";
		}
		
		if (mode === "login") {
			this.userIdContainer.innerHTML = "";
			this.link.innerHTML = this.messages.login;
			if (config.login) {
				this.link.href = config.login.href ? config.login.href : "#";
			} else {
				this.link.href = "#";	
			}
			this._clickEvent = this.connect(this.link, "onclick", function (e) {
				e.preventDefault();
				if (config.login && config.login.callback) {
					var doLogin = config.login.callback(e);
					if (doLogin !== false) {
						jazz.app.auth.login();
					}
				} else {
					jazz.app.auth.login();
				}
			});
			this.linkContainer.style.display = "inline";
		} else {
			if (config.user) {
				var userLabel = jazz.util.html.escape(config.user.name || config.user.userId);
				if (config.user.href ||	config.user.callback) {
					// Link
					var userIDLink = document.createElement("a");
					dojo.addClass(userIDLink, "link");
					userIDLink.innerHTML = userLabel;
					if (config.user.href) {
						// Href
						userIDLink.href = config.user.href;
					} else if (config.user.callback) {
						// Callback
						this._userCallbackHandler = this.connect(userIDLink, "onclick", function(event) {
							config.user.callback(config.user);
						});	
					}
					this.userIdContainer.appendChild(userIDLink);
				} else {
					// Text
					this.userIdContainer.innerHTML = userLabel;
				}
				
				if (config.user.menuProvider) {
					var chevronContainer = document.createElement("span");
					dojo.addClass(chevronContainer, "link");
					chevronContainer.innerHTML = "&nbsp;&nbsp;";
					this._userHoverMenu = new jazz.app.internal.HoverMenu({
						align: "right",
						menuClass: "jazz-app-Banner-menu", 
						menuProvider: config.user.menuProvider, 
						around: chevronContainer
					}, chevronContainer);
					
					this.userIdContainer.appendChild(chevronContainer);
				}
			}
			var logoutObj = config.logout;
			var formAuth = document.cookie.match("JazzFormAuth=") ? true : false;
			if (logoutObj || formAuth) { // show the logout link
				if (!logoutObj) logoutObj = {};
				this.linkContainer.style.display = "inline";
				dojo.mixin(this.link, {
					innerHTML: this.messages.logout,
					href: logoutObj.href ? logoutObj.href : "#"
				});
				if (logoutObj.callback || formAuth) { // we need to attach an onclick handler
					this._clickEvent = this.connect(this.link, "onclick", function(e) {
						if ((logoutObj.callback ? logoutObj.callback(e) : true) === false) { // if there's a callback defined and it returns boolean false, cancel the click
							e.preventDefault();
						} else if (formAuth) {
							e.preventDefault();
							if (!PlatformUI.getWorkbench().shouldVetoNavigation()) {  // check for dirty editors
								jazz.app.auth.logout().addCallback(function(){ 
									// On logout success, forward to specified href or reload the page (will show login screen)
									logoutObj.href ? (document.location = logoutObj.href) : document.location.reload();
								});
							}
						}
					});
				}
			} else {
				this.linkContainer.style.display = "none";
			}
		}
	}
});

})();