/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("jazz.app.internal.ResourceLinkAggregator");

dojo.require("jazz.app.proxy");

jazz.app.internal.resourceLinkAggreagtor = function () {
	if (!jazz.app.internal._resourceLinkAggregator) {
		jazz.app.internal._resourceLinkAggregator = new jazz.app.internal.ResourceLinkAggregator();
	}
	return jazz.app.internal._resourceLinkAggregator;
}

dojo.declare("jazz.app.internal.ResourceLinkAggregator", null, {
	
	constructor: function () {
		this.reset();
		this.resetOauth();
	},
	
	reset: function () {
		this._resourceLinks = {};
		this._timeoutId = null;
	},
	
	resetOauth: function (token) {
		if (token && this._oauthRLs) {
			this._oauthRLs[token] = [];
		} else {
			this._oauthRLs = {};
		}
	},
	
	/**
	 * A ResourceLink uses this method to register itself with the ResourceLinkAggregator.  The
	 * ResourceLinkAggregator collects information about all of the ResourceLinks that register,
	 * and it sends a batch request for compact rendering information after the page finishes
	 * loading.
	 * 
	 * @param {String} uri - The resource URI that is associated with the ResourceLink being registered
	 * @param {Function} callback - This function is called after a successful response
	 * @param {Function} errback - This function is called if there is an error response
	 */
	register: function (uri, callback, errback) {
		if (this._requestPending) {
			if (!this._registerQueue)
				this._registerQueue = [];
			this._registerQueue.push({uri: uri, callback: callback, errback: errback});
			return;
		}
		if (!this._resourceLinks[uri]) {
			this._resourceLinks[uri] = [
				{
					callback: callback,
					errback: errback
				}
			];
		} else {
			this._resourceLinks[uri].push({
				callback: callback,
				errback: errback
			});
		}
		
		if (!this._timeoutId) {
			// yield until the page finishes loading, allowing all the ResourceLinks on the
			// page to register before sending the batch request.
			this._timeoutId = setTimeout(dojo.hitch(this, this.sendRequest), 0);
		}
	},
		
	sendRequest: function () {
		this._requestPending = true;
		var uris = [];
		for (var i in this._resourceLinks) {
			uris.push(i);
		}
		
		var dfd = jazz.client.xhrPost({
			url: dojo.baseUrl + "../../_compactRendering",
			handleAs: "json",
			content: {
				uri: uris
			}
		});
		dfd.addCallback(dojo.hitch(this, this.handleResponse, null));
		dfd.addErrback(dojo.hitch(this, this.handleError, null));
	},
	
	handleResponse: function (token, resp) {
		var rlObj = token ? this._oauthRLs[token] : this._resourceLinks;
		for (var uri in rlObj) {
			if (resp[uri]) {
				for (var i = 0; i < rlObj[uri].length; i++) {
					if (resp[uri]._err) {
						rlObj[uri][i].errback(resp[uri]._err);
					}
					else {
						if (!token && resp[uri].oauthRequired) {
							if (this._oauthRLs[resp[uri].token] == null) {
								this._oauthRLs[resp[uri].token] = {};
							}
							if (this._oauthRLs[resp[uri].token][uri] == null) {
								this._oauthRLs[resp[uri].token][uri] = [];
							}
							this._oauthRLs[resp[uri].token][uri].push(rlObj[uri][i]);
						}
						rlObj[uri][i].callback(resp[uri]);
					}
				}
			}
		}
		// reset the status of the aggregator after sending back all the responses
		if (token) {
			this.resetOauth(token);
		} else {
			this.reset();
		}
		this._requestPending = false;
		if (this._registerQueue) {
			for (var i = 0; i < this._registerQueue.length; i++) {
				this.register(this._registerQueue[i].uri, this._registerQueue[i].callback, this._registerQueue[i].errback);
			}
		}
		delete this._registerQueue;
	},
	
	handleError: function (token, err) {
		for (var uri in this._resourceLinks) {
			for (var i=0; i < this._resourceLinks[uri].length; i++) {
				this._resourceLinks[uri][i].errback(err);
			}
		}
		// reset the status of the aggregator after sending back all the responses
		if (token) {
			this.resetOauth(token);
		} else {
			this.reset();
		}
	},
	
	resendOauth: function (token) {
		var uris = [];
		if (this._oauthRLs[token]) {
			for (var i in this._oauthRLs[token]) {
				uris.push(i);
			}			
			var dfd = jazz.client.xhrPost({
				url: dojo.baseUrl + "../../_compactRendering",
				handleAs: "json",
				content: {
					uri: uris
				}
			});
			dfd.addCallback(dojo.hitch(this, this.handleResponse, token));
			dfd.addErrback(dojo.hitch(this, this.handleError, token));
		}
	}
});
})();
