/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("net.jazz.ajax.service.ui.internal.userdata.EditHistory");

dojo.require("net.jazz.ajax.ui.MappingUI");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.Workbench");
dojo.require("net.jazz.ajax.service.ui.userdata.util");
dojo.require("net.jazz.ajax.service.ui.internal.userdata.UserDataEditor");

dojo.require("jazz.client.xhr");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var HISTORY_EDITOR = "net.jazz.ajax.EditHistory";

dojo.declare("net.jazz.ajax.service.ui.internal.userdata.EditHistory", [net.jazz.ajax.service.ui.internal.userdata.UserDataEditor], {
	constructor: function (args) {
		this._dataType = "History Item";
	},
	
	postCreate: function() {
		this.inherited(arguments);
		
		var pathInfo = net.jazz.ajax.ui.MappingUI._getPathInfo();
		var pathParts = pathInfo.split("/");
		var historyPath = "";
		for (var i=0; i < pathParts.length; i++) {
			if (pathParts[i].indexOf(HISTORY_EDITOR) >= 0) {
				historyPath = pathParts.slice(i+1, pathParts.length).join("/"); 
				break;
			}
		}
		
		var dfd = net.jazz.ajax.service.ui.userdata.util.getHistoryItem(historyPath);
		if (dfd) {
			dfd.addCallback(dojo.hitch(this, this.processResponse));
			dfd.addErrback(dojo.hitch(this, this.historyErrback));
		} else {
			this.populate();
		}
		this.connect(this.saveButton, "onclick", "handleSave");
	},
	
	historyErrback: function (err) {
		this.processResponseErr(err, "history");
	},
	
	handleSave: function (e) {
		e.target.disabled = true;
		var dfd = net.jazz.ajax.service.ui.userdata.util.addHistory({
			uri: this.userData.uri,
			title: this.titleField.value,
			comments: this.commentsField.value,
			tags: this.tagsField.value
		});
		dfd.addCallback(dojo.hitch(this, this.saveCallback));
		dfd.addErrback(dojo.hitch(this, this.saveErrback));
	},
	
	saveCallback: function (response) {
		this.userData = {
			uri: this.userData.uri,
			title: this.titleField.value,
			comments: this.commentsField.value,
			tags: this.tagsField.value
		};
		this.populate();
		return response;
	},
	
	saveErrback: function(err) {
		// TODO - better error message
		alert("Error saving changes: " + dojo.toJson(err));
		return err;
	}
});
})();